const base64 = require('../functions/Base64');
const updaterModel = require("../model/updaterModel");
const updaterProdFiliModel = require("../model/updaterProdFiliModel");
const GeraNovoCodigo = require("../modules/NovoCodigoModule");
const OracleService = require("./OracleService");
const { v4: uuidv4 } = require("uuid");
const Base64 = require("../functions/Base64");

class UpdaterService {
  async insertEventTable(cdFilial, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi registrado no sistema.",
    };
   
    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length === 0) {
      const eventSeq = await GeraNovoCodigo.prxUpdaterEvento(cdFilial);
      const uuid = uuidv4();

      params = {
        P_CDFILIAL: cdFilial,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDPRODUCTID: devopsId,
        P_CDPRODFILIID: prodFiliId,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.insertPendingEvent(params);

      const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

      params = {
        P_CDFILIAL: cdFilial,
        P_CDHISTORICO: historySeq,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDOPERADOR: null,
        P_CDPRODUCTID: devopsId,
        P_NMPRODUCT: null,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.appendUpdaterHistory(params);

      response = {
        error: false,
        message: "Evento cadastrado com sucesso.",
      };

      await OracleService.commit();
    }

    return response;
  }

  async updateEventTable(cdFilial, devopsId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    const eventId = action === "INSTALACAO_EM_CURSO" ? "INSTALACAO_PENDENTE" : "REMOCAO_PENDENTE";

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
      P_IDSTEVENTO: eventId,
    };

    const pendingEvents = await updaterModel.getProductUpdaterEventByEventId(params);

    if (pendingEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: action,
        P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: null,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: action,
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async finishEventTable(cdFilial, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const inProgressEvents = await updaterModel.getInProgressEvents(params);

    if (inProgressEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "FINALIZADO",
        P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: null,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "FINALIZADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        await this.updateProdFiliVersion(prodFiliId, installVersion);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }

      await OracleService.commit();
    }

    return response;
  }

  async updateProdFiliVersion(prodFiliId, updateVersion) {
    const params = {
      NRVERSAO: updateVersion,
      ID: prodFiliId,
    };

    await updaterProdFiliModel.updateUpdaterProdFiliVersion(params);
  }

  async cancelUnitEvents(cdFilial) {
    let params = {
      P_CDFILIAL: cdFilial,
    };

    await updaterModel.cancelUnitEvents(params);

    await OracleService.commit();
  }

  async cancelEventTable(cdFilial, devopsId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "CANCELADO",
        P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: null,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "CANCELADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async getPendingEvents() {
    return updaterModel.getPendingEvents();
  }

  async getPendingEventsByUnit(cdFilial) {
    const params = {
      P_CDFILIAL: cdFilial,
    };

    return updaterModel.getPendingEventsByUnit(params);
  }
  
  async checkVersion(params) {
    const result = await updaterModel.getCheckInstalledVersion(params);
    return { error: result.rows.length > 0 };
  }

  async saveConfig({ config }){
    try {
      if (!config) {
        throw 'Codigo da Filial não foi informado';
      }

      const params = {
        P_CDFILIAL: config.environment.CDFILIAL
      };
      
      let configIni = await updaterModel.getConfigIni(params);

      if (configIni.rows.length > 0) {
        throw `Arquivo config.ini da Filial ${params.P_CDFILIAL} já está salvo na base de dados.`
      };
      
      const configIniBase64 = Base64.encode(config);         
      
      if (!configIniBase64) {
        throw 'Não foi possivel converter config.ini da filial para base64.';
      }
      
      params.P_CONFIG = configIniBase64;

      const insertConfig = await updaterModel.insertConfigIni(params);

      if (insertConfig.rowsAffected === 0) {
        throw 'Erro ao inserir as informações do config.ini na base de dados.'
      }

      configIni = await updaterModel.getConfigIni(params);

      if (!configIni) {
        throw 'Não foi possivel recuperar as informações do config.ini da filial.';
      }

      const operator = await updaterModel.getDefaultOperator(params);

      if (operator.rows.length === 0) {
        throw `Não foi possivel buscar o operador padrão da filial ${params.P_CDFILIAL}.`;
      }

      const cdOperador = operator.rows[0].CDOPERADOR;
      const updaterConfigId = configIni.rows[0].ID;

      if(!cdOperador && !updaterConfigId){
        throw 'Não foi possivel inserir os dados na tabela Historico';
      }

      params.P_CDOPERADOR = cdOperador;
      params.P_UPDATERCONFIGID = updaterConfigId;

      await updaterModel.appendConfigIniHistory(params);         

      await OracleService.commit();       
      
      return {
        error: false,
        message: `Config.ini da filial ${params.P_CDFILIAL} foi inserido com sucesso`
      }
    } catch(error) {     
      await OracleService.rollback();      
      
      return {
        error: true,
        message: error
      }
    }     
  }

  async getUnitConfigIniData(params) {
    try {
      const result = await updaterModel.getConfigIniByUnit(params);
  
      if (result.rows.length === 0) {
        throw new Error(`Não existe um arquivo de configuração salvo para a filial ${params.P_CDFILIAL}`);
      }

      return result.rows[0];
    } catch (error) {
      throw error;
    }
  }

  async getConfigIniByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };

      const result = await this.getUnitConfigIniData(params);
  
      const decodeConfig = base64.decode(result.CONFIG);
      
      return {
        error: false,
        message: 'Arquivo de configuração carregado com sucesso!',
        data: decodeConfig
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  async saveConfigIniByUnit(cdFilial, cdOperador, config) {
    try {
      const encodeConfig = base64.encode(config);
      let result;

      let params = {
        P_CDFILIAL: cdFilial,
        P_CDOPERADOR: cdOperador,
        P_CONFIG: encodeConfig
      }
      
      result = await updaterModel.updateConfigIniByUnit(params);

      if (result.rowsAffected === 0) {
        throw new Error(`Não existe registro de um arquivo de configuração para a filial ${cdFilial}.`);
      }

      result = await this.getUnitConfigIniData(params);

      params.P_UPDATERCONFIGID = result.ID;

      result = await updaterModel.appendConfigIniHistory(params);
      
      if (result.rowsAffected === 0) {
        throw new Error(`Falha ao salvar o histórico para o arquivo de configuração da filial ${cdFilial}.`);
      }

      await OracleService.commit();
      
      return {
        error: false,
        message: 'Arquivo de configuração salvo com sucesso!'
      }
    } catch (error) {
      await OracleService.rollback();
      
      return {
        error: true,
        message: 'Erro ao salvar o arquivo de configuração. Error: ' + error.message
      }
    }
  }

  async getConfigHistoryByUnit(cdFilial) {
    try {
      let result;
      const params = {
        P_CDFILIAL: cdFilial,
      };

      result = await this.getUnitConfigIniData(params);
      
      params.P_ID = result.ID;
      
      result = await updaterModel.getConfigIniHistoryByUnit(params);
      
      return {
        error: false,
        message: 'Histórico de atualização do arquivo de configuração carregado com sucesso!',
        data: result.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }
}

module.exports = new UpdaterService();
